#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Models for storing data from Campaign Disclosure Statements (Form 460).
"""
from django.db import models
from calaccess_processed_filings.models.campaign import CampaignFinanceFilingBase


class Form460FilingBase(CampaignFinanceFilingBase):
    """
    Base and abstract model for Form 460 filings.
    """

    statement_type = models.CharField(
        max_length=50,
        verbose_name="statement type",
        help_text='Type of statement, e.g., "Quarterly", "Semi-Annual", Pre-'
        "Election (from CVR_CAMPAIGN_DISCLOSURE.STMT_TYPE)",
    )
    from_date = models.DateField(
        verbose_name="from date",
        db_index=True,
        null=False,
        help_text="The first date of the filing period covered by the statement "
        "(from CVR_CAMPAIGN_DISCLOSURE.FROM_DATE)",
    )
    thru_date = models.DateField(
        verbose_name="thru date",
        db_index=True,
        null=False,
        help_text="The last date of the filing period covered by the statement "
        "(from CVR_CAMPAIGN_DISCLOSURE.THRU_DATE)",
    )
    monetary_contributions = models.IntegerField(
        verbose_name="monetary contributions",
        null=True,
        help_text="Total monetary contributions (from line 1, column A)",
    )
    loans_received = models.IntegerField(
        verbose_name="loans received",
        null=True,
        help_text="Total loans received (from line 2, column A)",
    )
    subtotal_cash_contributions = models.IntegerField(
        verbose_name="subtotal cash contributions",
        null=True,
        help_text="Monetary contributions and loans received combined (from "
        "line 3, column A)",
    )
    nonmonetary_contributions = models.IntegerField(
        verbose_name="nonmonetary contributions",
        null=True,
        help_text="Non-monetary contributions (from line 4, column A)",
    )
    total_contributions = models.IntegerField(
        verbose_name="total contributions",
        null=True,
        help_text="Total contributions (from line 5, column A)",
    )
    payments_made = models.IntegerField(
        verbose_name="payments made",
        null=True,
        help_text="Payments made (from line 6, column A)",
    )
    loans_made = models.IntegerField(
        verbose_name="loans made",
        null=True,
        help_text="Loans made (from line 7, column A)",
    )
    subtotal_cash_payments = models.IntegerField(
        verbose_name="subtotal cash payments",
        null=True,
        help_text="Sub-total of cash payments (from line 8, column A)",
    )
    unpaid_bills = models.IntegerField(
        verbose_name="unpaid bills",
        null=True,
        help_text="Unpaid bills / accrued expenses (from line 9, column A)",
    )
    nonmonetary_adjustment = models.IntegerField(
        verbose_name="nonmonetary adjustment",
        null=True,
        help_text="Non-monetary adjustment (from line 10, column A), which is "
        "equal to the total of non-monetary contributions",
    )
    total_expenditures_made = models.IntegerField(
        verbose_name="total expenditures made",
        null=True,
        help_text="Total expenditures made (from line 11, column A)",
    )
    beginning_cash_balance = models.IntegerField(
        verbose_name="begin cash balance",
        null=True,
        help_text="Beginning cash balance (from line 12), which is equal to "
        "the Ending Cash Balance (line 16) reported on the summary "
        "page of the previous Form 460 filing",
    )
    cash_receipts = models.IntegerField(
        verbose_name="cash receipts",
        null=True,
        help_text="Cash receipts (from line 13)",
    )
    miscellaneous_cash_increases = models.IntegerField(
        verbose_name="miscellaneous cash increases",
        null=True,
        help_text="Miscellaneous cash increases (from line 14)",
    )
    cash_payments = models.IntegerField(
        verbose_name="cash payments",
        null=True,
        help_text="Cash payments (from line 15)",
    )
    ending_cash_balance = models.IntegerField(
        verbose_name="ending cash balance",
        null=True,
        help_text="Ending cash balance (from line 16)",
    )
    loan_guarantees_received = models.IntegerField(
        verbose_name="loan guarantees received",
        null=True,
        help_text="Loan guarantees received (from line 17)",
    )
    cash_equivalents = models.IntegerField(
        verbose_name="cash equivalents",
        null=True,
        help_text="Cash equivalents (from line 18), which includes investments "
        "that can't be readily converted to cash, such as outstanding "
        "loans the committee has made to others",
    )
    outstanding_debts = models.IntegerField(
        verbose_name="outstanding debts",
        null=True,
        help_text="Outstanding debts on loans owed by the committee (from line " "19)",
    )

    class Meta:
        """
        Model options.
        """

        app_label = "calaccess_processed_filings"
        abstract = True
